@@ -3,6 +3,7 @@ from pytest import raises
 from trilogy.core.exceptions import UndefinedConceptException
 from trilogy.core.models.environment import Environment
 from trilogy.parsing.parse_engine import (
+    ERROR_CODES,
     PARSER,
     InvalidSyntaxException,
     ParseToObjects,
@@ -66,18 +67,57 @@ address fun;
     x.run_second_parse_pass()
 
 
-TEXT2 = """
+def test_from_error():
+    env = Environment()
+    TEXT2 = """
+    const a <- 1;
+
+    select
+        a,
+    FROM a
+    ;
+    """
+    with raises(InvalidSyntaxException) as e:
+        env.parse(TEXT2)
+    assert ERROR_CODES[101] in str(e.value), e.value
+
+
+def test_error_order_by_missing():
+    env = Environment()
+    TEXT2 = """
 const a <- 1;
 
 select
     a,
-FROM a
+order by a
 ;
-"""
-
-
-def test_from_error():
-    env = Environment()
-
-    with raises(InvalidSyntaxException):
+    """
+    with raises(InvalidSyntaxException) as e:
         env.parse(TEXT2)
+    assert ERROR_CODES[210] in str(e.value)
+
+    env = Environment()
+    TEXT2 = """
+const a <- 1;
+
+select
+    a,
+order by a;
+    """
+    with raises(InvalidSyntaxException) as e:
+        env.parse(TEXT2)
+    assert ERROR_CODES[210] in str(e.value)
+
+
+def test_alias_error():
+    env = Environment()
+    TEXT2 = """
+    const a <- 1;
+
+    select
+        a+2 fun,
+    ;
+    """
+    with raises(InvalidSyntaxException) as e:
+        env.parse(TEXT2)
+    assert ERROR_CODES[201] in str(e.value), e.value
