@@ -154,6 +154,7 @@ class FunctionType(Enum):
     ARRAY_SUM = "array_sum"
     ARRAY_SORT = "array_sort"
     ARRAY_TRANSFORM = "array_transform"
+    ARRAY_TO_STRING = "array_to_string"
 
     # TEXT AND MAYBE MORE
     SPLIT = "split"
@@ -171,6 +172,7 @@ class FunctionType(Enum):
     RANDOM = "random"
     FLOOR = "floor"
     CEIL = "ceil"
+    LOG = "log"
 
     # Aggregates
     ## group is not a real aggregate - it just means group by this + some other set of fields
@@ -193,6 +195,8 @@ class FunctionType(Enum):
     SUBSTRING = "substring"
     STRPOS = "strpos"
     CONTAINS = "contains"
+    TRIM = "trim"
+    REPLACE = "replace"
 
     # STRING REGEX
     REGEXP_CONTAINS = "regexp_contains"
@@ -353,7 +357,7 @@ class SourceType(Enum):
 
 
 class ShowCategory(Enum):
-    MODELS = "models"
+    DATASOURCES = "datasources"
     CONCEPTS = "concepts"
 
 
