@@ -283,6 +283,15 @@ FUNCTION_REGISTRY: dict[FunctionType, FunctionConfig] = {
         output_type_function=get_transform_output_type,
         arg_count=3,
     ),
+    FunctionType.ARRAY_TO_STRING: FunctionConfig(
+        valid_inputs={
+            DataType.ARRAY,
+            DataType.STRING,
+        },
+        output_purpose=Purpose.PROPERTY,
+        output_type=DataType.STRING,
+        arg_count=2,
+    ),
     FunctionType.ARRAY_SUM: FunctionConfig(
         valid_inputs={
             DataType.ARRAY,
@@ -358,6 +367,16 @@ FUNCTION_REGISTRY: dict[FunctionType, FunctionConfig] = {
         output_type=DataType.INTEGER,
         arg_count=2,
     ),
+    FunctionType.REPLACE: FunctionConfig(
+        valid_inputs=[
+            {DataType.STRING},
+            {DataType.STRING},
+            {DataType.STRING},
+        ],
+        output_purpose=Purpose.PROPERTY,
+        output_type=DataType.STRING,
+        arg_count=3,
+    ),
     FunctionType.CONTAINS: FunctionConfig(
         valid_inputs=[
             {DataType.STRING},
@@ -367,6 +386,15 @@ FUNCTION_REGISTRY: dict[FunctionType, FunctionConfig] = {
         output_type=DataType.BOOL,
         arg_count=2,
     ),
+    FunctionType.TRIM: FunctionConfig(
+        valid_inputs=[
+            {DataType.STRING},
+            {DataType.STRING},
+        ],
+        output_purpose=Purpose.PROPERTY,
+        output_type=DataType.BOOL,
+        arg_count=2,
+    ),
     FunctionType.SUBSTRING: FunctionConfig(
         valid_inputs=[{DataType.STRING}, {DataType.INTEGER}, {DataType.INTEGER}],
         output_purpose=Purpose.PROPERTY,
@@ -688,6 +716,15 @@ FUNCTION_REGISTRY: dict[FunctionType, FunctionConfig] = {
         output_type=DataType.INTEGER,
         arg_count=1,
     ),
+    FunctionType.LOG: FunctionConfig(
+        valid_inputs=[
+            {DataType.INTEGER, DataType.FLOAT, DataType.NUMBER, DataType.NUMERIC},
+            {DataType.INTEGER},
+        ],
+        output_purpose=Purpose.PROPERTY,
+        output_type=DataType.FLOAT,
+        arg_count=2,
+    ),
     FunctionType.RANDOM: FunctionConfig(
         valid_inputs=[],
         output_purpose=Purpose.PROPERTY,
@@ -775,6 +812,7 @@ FUNCTION_REGISTRY: dict[FunctionType, FunctionConfig] = {
             DataType.FLOAT,
             DataType.NUMBER,
             DataType.NUMERIC,
+            DataType.BOOL,
         },
         output_purpose=Purpose.METRIC,
         arg_count=1,
