@@ -77,7 +77,7 @@ class Mergeable(ABC):
     def with_merge(self, source: Concept, target: Concept, modifiers: List[Modifier]):
         raise NotImplementedError
 
-    def with_reference_replacement(self, source: str, target: Expr):
+    def with_reference_replacement(self, source: str, target: Expr | ArgBinding):
         raise NotImplementedError(type(self))
 
 
@@ -161,7 +161,7 @@ class ConceptRef(Addressable, Namespaced, DataTyped, Mergeable, BaseModel):
             metadata=self.metadata,
         )
 
-    def with_reference_replacement(self, source: str, target: Expr):
+    def with_reference_replacement(self, source: str, target: Expr | ArgBinding):
         if self.address == source:
             return target
         return self
@@ -1605,6 +1605,10 @@ def get_concept_arguments(expr) -> List["ConceptRef"]:
     return output
 
 
+def args_to_pretty(input: set[DataType]) -> str:
+    return ", ".join(sorted([f"'{x.value}'" for x in input if x != DataType.UNKNOWN]))
+
+
 class Function(DataTyped, ConceptArgs, Mergeable, Namespaced, BaseModel):
     operator: FunctionType
     arg_count: int = Field(default=1)
@@ -1669,9 +1673,10 @@ class Function(DataTyped, ConceptArgs, Mergeable, Namespaced, BaseModel):
                 and get_basic_type(arg.datatype.data_type) not in valid_inputs[idx]
             ):
                 if arg.datatype != DataType.UNKNOWN:
+
                     raise TypeError(
-                        f"Invalid input datatype {arg.datatype.data_type} passed into position {idx}"
-                        f" for {operator_name} from concept {arg.name}, valid is {valid_inputs[idx]}"
+                        f"Invalid argument type '{arg.datatype.data_type.value}' passed into {operator_name} function in position {idx+1}"
+                        f" from concept: {arg.name}. Valid: {args_to_pretty(valid_inputs[idx])}."
                     )
             if (
                 isinstance(arg, Function)
@@ -1679,8 +1684,8 @@ class Function(DataTyped, ConceptArgs, Mergeable, Namespaced, BaseModel):
             ):
                 if arg.output_datatype != DataType.UNKNOWN:
                     raise TypeError(
-                        f"Invalid input datatype {arg.output_datatype} passed into"
-                        f" {operator_name} from function {arg.operator.name}, need {valid_inputs[idx]}"
+                        f"Invalid argument type {arg.output_datatype}' passed into"
+                        f" {operator_name} function from function {arg.operator.name} in position {idx+1}. Valid: {args_to_pretty(valid_inputs[idx])}"
                     )
             # check constants
             comparisons: List[Tuple[Type, DataType]] = [
@@ -1708,7 +1713,7 @@ class Function(DataTyped, ConceptArgs, Mergeable, Namespaced, BaseModel):
                     )
         return v
 
-    def with_reference_replacement(self, source: str, target: Expr):
+    def with_reference_replacement(self, source: str, target: Expr | ArgBinding):
         from trilogy.core.functions import arg_to_datatype, merge_datatypes
 
         nargs = [
@@ -2303,13 +2308,13 @@ class CustomFunctionFactory:
         ]
         return self
 
-    def __call__(self, *creation_args: Expr):
+    def __call__(self, *creation_args: ArgBinding | Expr):
         nout = (
             self.function.model_copy(deep=True)
             if isinstance(self.function, BaseModel)
             else self.function
         )
-        creation_arg_list: list[Expr] = list(creation_args)
+        creation_arg_list: list[ArgBinding | Expr] = list(creation_args)
         if len(creation_args) < len(self.function_arguments):
             for binding in self.function_arguments[len(creation_arg_list) :]:
                 if binding.default is None:
@@ -2361,7 +2366,9 @@ class Comment(BaseModel):
 class ArgBinding(Namespaced, DataTyped, BaseModel):
     name: str
     default: Expr | None = None
-    datatype: DataType = DataType.UNKNOWN
+    datatype: (
+        DataType | MapType | ArrayType | NumericType | StructType | TraitDataType
+    ) = DataType.UNKNOWN
 
     def with_namespace(self, namespace):
         return ArgBinding(
