@@ -16,6 +16,7 @@ from trilogy.core.enums import (
     DatePart,
     FunctionType,
     Ordering,
+    ShowCategory,
     UnnestMode,
     WindowType,
 )
@@ -77,6 +78,7 @@ from trilogy.core.statements.execute import (
     ProcessedQueryPersist,
     ProcessedRawSQLStatement,
     ProcessedShowStatement,
+    ProcessedStaticValueOutput,
 )
 from trilogy.core.utility import safe_quote
 from trilogy.dialect.common import render_join, render_unnest
@@ -191,6 +193,9 @@ FUNCTION_MAP = {
     FunctionType.ARRAY_TRANSFORM: lambda args: (
         f"array_transform({args[0]}, {args[1]} -> {args[2]})"
     ),
+    FunctionType.ARRAY_TO_STRING: lambda args: (
+        f"array_to_string({args[0]}, {args[1]})"
+    ),
     # math
     FunctionType.ADD: lambda x: " + ".join(x),
     FunctionType.ABS: lambda x: f"abs({x[0]})",
@@ -203,6 +208,9 @@ FUNCTION_MAP = {
     FunctionType.MOD: lambda x: f"({x[0]} % {x[1]})",
     FunctionType.SQRT: lambda x: f"sqrt({x[0]})",
     FunctionType.RANDOM: lambda x: "random()",
+    FunctionType.LOG: lambda x: (
+        f"log({x[0]})" if x[1] == 10 else f"log({x[0]}, {x[1]})"
+    ),
     # aggregate types
     FunctionType.COUNT_DISTINCT: lambda x: f"count(distinct {x[0]})",
     FunctionType.COUNT: lambda x: f"count({x[0]})",
@@ -222,6 +230,8 @@ FUNCTION_MAP = {
     FunctionType.REGEXP_CONTAINS: lambda x: f"REGEXP_CONTAINS({x[0]},{x[1]})",
     FunctionType.REGEXP_EXTRACT: lambda x: f"REGEXP_EXTRACT({x[0]},{x[1]})",
     FunctionType.REGEXP_REPLACE: lambda x: f"REGEXP_REPLACE({x[0]},{x[1]}, {x[2]})",
+    FunctionType.TRIM: lambda x: f"TRIM({x[0]})",
+    FunctionType.REPLACE: lambda x: f"REPLACE({x[0]},{x[1]},{x[2]})",
     # FunctionType.NOT_LIKE: lambda x: f" CASE WHEN {x[0]} like {x[1]} THEN 0 ELSE 1 END",
     # date types
     FunctionType.DATE_TRUNCATE: lambda x: f"date_trunc({x[0]},{x[1]})",
@@ -638,18 +648,7 @@ class BaseDialect:
             ):
                 return f"{self.render_expr(e.left, cte=cte, cte_map=cte_map, raise_invalid=raise_invalid)} {e.operator.value} {self.render_expr(e.right, cte=cte, cte_map=cte_map, raise_invalid=raise_invalid)}"
 
-            elif isinstance(
-                e.right,
-                (
-                    str,
-                    int,
-                    bool,
-                    float,
-                ),
-            ):
-                return f"{self.render_expr(e.left, cte=cte, cte_map=cte_map, raise_invalid=raise_invalid)} {e.operator.value} ({self.render_expr(e.right, cte=cte, cte_map=cte_map, raise_invalid=raise_invalid)})"
-            else:
-                return f"{self.render_expr(e.left, cte=cte, cte_map=cte_map, raise_invalid=raise_invalid)} {e.operator.value} {self.render_expr(e.right, cte=cte, cte_map=cte_map, raise_invalid=raise_invalid)}"
+            return f"{self.render_expr(e.left, cte=cte, cte_map=cte_map, raise_invalid=raise_invalid)} {e.operator.value} ({self.render_expr(e.right, cte=cte, cte_map=cte_map, raise_invalid=raise_invalid)})"
         elif isinstance(e, COMPARISON_ITEMS):
             return f"{self.render_expr(e.left, cte=cte, cte_map=cte_map, raise_invalid=raise_invalid)} {e.operator.value} {self.render_expr(e.right, cte=cte, cte_map=cte_map, raise_invalid=raise_invalid)}"
         elif isinstance(e, CONDITIONAL_ITEMS):
@@ -953,6 +952,46 @@ class BaseDialect:
             self.render_cte(sort_select_output(query.ctes[-1], query), auto_sort=False)
         ]
 
+    def create_show_output(
+        self,
+        environment: Environment,
+        content: ShowCategory,
+    ):
+        if content == ShowCategory.CONCEPTS:
+            output_columns = [
+                environment.concepts[
+                    DEFAULT_CONCEPTS["concept_address"].address
+                ].reference,
+                environment.concepts[
+                    DEFAULT_CONCEPTS["concept_datatype"].address
+                ].reference,
+                environment.concepts[
+                    DEFAULT_CONCEPTS["concept_description"].address
+                ].reference,
+            ]
+            output_values = [
+                {
+                    DEFAULT_CONCEPTS["concept_address"].address: (
+                        concept.name
+                        if concept.namespace == DEFAULT_NAMESPACE
+                        else concept.address
+                    ),
+                    DEFAULT_CONCEPTS["concept_datatype"].address: str(concept.datatype),
+                    DEFAULT_CONCEPTS[
+                        "concept_description"
+                    ].address: concept.metadata.description
+                    or "",
+                }
+                for _, concept in environment.concepts.items()
+                if not concept.is_internal
+            ]
+        else:
+            raise NotImplementedError(f"Show category {content} not implemented")
+        return ProcessedShowStatement(
+            output_columns=output_columns,
+            output_values=[ProcessedStaticValueOutput(values=output_values)],
+        )
+
     def generate_queries(
         self,
         environment: Environment,
@@ -1027,8 +1066,12 @@ class BaseDialect:
                             ],
                         )
                     )
+                elif isinstance(statement.content, ShowCategory):
+                    output.append(
+                        self.create_show_output(environment, statement.content)
+                    )
                 else:
-                    raise NotImplementedError(type(statement))
+                    raise NotImplementedError(type(statement.content))
             elif isinstance(statement, RawSQLStatement):
                 output.append(ProcessedRawSQLStatement(text=statement.text))
             elif isinstance(
