@@ -7,7 +7,7 @@ from pathlib import Path
 from re import IGNORECASE
 from typing import Any, List, Optional, Tuple, Union
 
-from lark import Lark, ParseTree, Transformer, Tree, v_args
+from lark import Lark, ParseTree, Token, Transformer, Tree, v_args
 from lark.exceptions import (
     UnexpectedCharacters,
     UnexpectedEOF,
@@ -1793,10 +1793,18 @@ class ParseToObjects(Transformer):
     def fstrpos(self, meta, args):
         return self.function_factory.create_function(args, FunctionType.STRPOS, meta)
 
+    @v_args(meta=True)
+    def freplace(self, meta, args):
+        return self.function_factory.create_function(args, FunctionType.REPLACE, meta)
+
     @v_args(meta=True)
     def fcontains(self, meta, args):
         return self.function_factory.create_function(args, FunctionType.CONTAINS, meta)
 
+    @v_args(meta=True)
+    def ftrim(self, meta, args):
+        return self.function_factory.create_function(args, FunctionType.TRIM, meta)
+
     @v_args(meta=True)
     def fsubstring(self, meta, args):
         return self.function_factory.create_function(args, FunctionType.SUBSTRING, meta)
@@ -1971,6 +1979,12 @@ class ParseToObjects(Transformer):
             args.append(0)
         return self.function_factory.create_function(args, FunctionType.ROUND, meta)
 
+    @v_args(meta=True)
+    def flog(self, meta, args) -> Function:
+        if len(args) == 1:
+            args.append(10)
+        return self.function_factory.create_function(args, FunctionType.LOG, meta)
+
     @v_args(meta=True)
     def ffloor(self, meta, args) -> Function:
         return self.function_factory.create_function(args, FunctionType.FLOOR, meta)
@@ -2020,6 +2034,12 @@ class ParseToObjects(Transformer):
             args, FunctionType.ARRAY_DISTINCT, meta
         )
 
+    @v_args(meta=True)
+    def farray_to_string(self, meta, args):
+        return self.function_factory.create_function(
+            args, FunctionType.ARRAY_TO_STRING, meta
+        )
+
     @v_args(meta=True)
     def farray_sort(self, meta, args):
         if len(args) == 1:
@@ -2034,12 +2054,17 @@ class ParseToObjects(Transformer):
         return self.environment.functions[args[0]]
 
     @v_args(meta=True)
-    def farray_transform(self, meta, args):
+    def farray_transform(self, meta, args) -> Function:
         factory: CustomFunctionFactory = args[1]
         if not len(factory.function_arguments) == 1:
             raise InvalidSyntaxException(
                 "Array transform function must have exactly one argument;"
             )
+        array_type = arg_to_datatype(args[0])
+        if not isinstance(array_type, ArrayType):
+            raise InvalidSyntaxException(
+                f"Array transform function must be applied to an array, not {array_type}"
+            )
         return self.function_factory.create_function(
             [
                 args[0],
@@ -2047,7 +2072,7 @@ class ParseToObjects(Transformer):
                 factory(
                     ArgBinding(
                         name=factory.function_arguments[0].name,
-                        datatype=arg_to_datatype(args[0]).value_data_type,
+                        datatype=array_type.value_data_type,
                     )
                 ),
             ],
@@ -2069,9 +2094,9 @@ def unpack_visit_error(e: VisitError, text: str | None = None):
                 extract = text[e.obj.meta.start_pos - 5 : e.obj.meta.end_pos + 5]
                 raise InvalidSyntaxException(
                     str(e.orig_exc)
-                    + " in "
+                    + " Raised when parsing rule: "
                     + str(e.rule)
-                    + f" Line: {e.obj.meta.line} ({extract})"
+                    + f' Line: {e.obj.meta.line} "...{extract}..."'
                 )
             InvalidSyntaxException(
                 str(e.orig_exc) + " in " + str(e.rule) + f" Line: {e.obj.meta.line}"
@@ -2086,6 +2111,53 @@ def parse_text_raw(text: str, environment: Optional[Environment] = None):
     PARSER.parse(text)
 
 
+ERROR_CODES: dict[int, str] = {
+    # 100 code are SQL compatability errors
+    101: "Using FROM keyword? Trilogy does not have a FROM clause (Datasource resolution is automatic).",
+    # 200 codes relate to required explicit syntax (we could loosen these?)
+    201: 'Missing alias? Alias must be specified with "AS" - e.g. `SELECT x+1 AS y`',
+    210: "Missing order direction? Order by must be explicit about direction - specify `asc` or `desc`.",
+}
+
+DEFAULT_ERROR_SPAN: int = 30
+
+
+def inject_context_maker(pos: int, text: str, span: int = 40) -> str:
+    """Returns a pretty string pinpointing the error in the text,
+    with span amount of context characters around it.
+
+    Note:
+        The parser doesn't hold a copy of the text it has to parse,
+        so you have to provide it again
+    """
+
+    start = max(pos - span, 0)
+    end = pos + span
+    if not isinstance(text, bytes):
+
+        before = text[start:pos].rsplit("\n", 1)[-1]
+        after = text[pos:end].split("\n", 1)[0]
+        rcap = ""
+        # if it goes beyond the end of text, no ...
+        # if it terminates on a space, no need for ...
+        if not after[-1].isspace() and not (end > len(text)):
+            rcap = "..."
+        lcap = ""
+        if start > 0 and not before[0].isspace():
+            lcap = "..."
+        lpad = " "
+        rpad = " "
+        if before.endswith(" "):
+            lpad = ""
+        if after.startswith(" "):
+            rpad = ""
+        return f"{lcap}{before}{lpad}???{rpad}{after}{rcap}"
+    else:
+        before = text[start:pos].rsplit(b"\n", 1)[-1]
+        after = text[pos:end].split(b"\n", 1)[0]
+        return (before + b" ??? " + after).decode("ascii", "backslashreplace")
+
+
 def parse_text(
     text: str,
     environment: Optional[Environment] = None,
@@ -2103,6 +2175,48 @@ def parse_text(
         | None
     ],
 ]:
+    def _create_syntax_error(code: int, pos: int, text: str) -> InvalidSyntaxException:
+        """Helper to create standardized syntax error with context."""
+        return InvalidSyntaxException(
+            f"Syntax [{code}]: "
+            + ERROR_CODES[code]
+            + "\nLocation:\n"
+            + inject_context_maker(pos, text.replace("\n", " "), DEFAULT_ERROR_SPAN)
+        )
+
+    def _create_generic_syntax_error(
+        message: str, pos: int, text: str
+    ) -> InvalidSyntaxException:
+        """Helper to create generic syntax error with context."""
+        return InvalidSyntaxException(
+            message
+            + "\nLocation:\n"
+            + inject_context_maker(pos, text.replace("\n", " "), DEFAULT_ERROR_SPAN)
+        )
+
+    def _handle_unexpected_token(e: UnexpectedToken, text: str) -> None:
+        """Handle UnexpectedToken errors with specific logic."""
+        # Handle ordering direction error
+        pos = e.pos_in_stream or 0
+        if e.expected == {"ORDERING_DIRECTION"}:
+            raise _create_syntax_error(210, pos, text)
+
+        # Handle FROM token error
+        parsed_tokens = [x.value for x in e.token_history] if e.token_history else []
+        if parsed_tokens == ["FROM"]:
+            raise _create_syntax_error(101, pos, text)
+
+        # Attempt recovery for aliasing
+        try:
+            e.interactive_parser.feed_token(Token("AS", "AS"))
+            e.interactive_parser.feed_token(Token("IDENTIFIER", e.token.value))
+            raise _create_syntax_error(201, pos, text)
+        except UnexpectedToken:
+            pass
+
+        # Default UnexpectedToken handling
+        raise _create_generic_syntax_error(str(e), pos, text)
+
     environment = environment or (
         Environment(working_path=root) if root else Environment()
     )
@@ -2110,6 +2224,7 @@ def parse_text(
         environment=environment, import_keys=["root"], parse_config=parse_config
     )
     start = datetime.now()
+
     try:
         parser.set_text(text)
         # disable fail on missing to allow for circular dependencies
@@ -2127,20 +2242,11 @@ def parse_text(
         unpack_visit_error(e, text)
         # this will never be reached
         raise e
-    except (
-        UnexpectedCharacters,
-        UnexpectedEOF,
-        UnexpectedInput,
-        UnexpectedToken,
-        ValidationError,
-        TypeError,
-    ) as e:
-        if isinstance(
-            e, (UnexpectedCharacters, UnexpectedEOF, UnexpectedInput, UnexpectedToken)
-        ):
-            raise InvalidSyntaxException(
-                str(e) + "\nContext:\n" + e.get_context(text.replace("\n", " "), 20)
-            )
+    except UnexpectedToken as e:
+        _handle_unexpected_token(e, text)
+    except (UnexpectedCharacters, UnexpectedEOF, UnexpectedInput) as e:
+        raise _create_generic_syntax_error(str(e), e.pos_in_stream or 0, text)
+    except (ValidationError, TypeError) as e:
         raise InvalidSyntaxException(str(e))
 
     return environment, output
